﻿using System.Collections.Generic;
using Core.Domain;
using Infrastructure;
using NUnit.Framework;

namespace Test.Unit.ACVSimulator.Core
{
    public class HeadTest
    {
        private Head head;
        private Dictionary<BrainSector, HeadPiece> headPieces = new Dictionary<BrainSector, HeadPiece>();
        private HeadPiece leftEye;
    
        [SetUp]
        public void SetUp()
        {    
            headPieces.Clear();
        }

        [Test]
        public void touch_head_piece_mapped_to_brain_part()
        {
            GivenThereIsALeftEyeRelatedWith(BrainSector.Center);
            GivenThereIsAHeadBasedOnConfig();
        
            WhenTheBrainIsTouchedIn(BrainSector.Center);
        
            ThenTheLeftEyeIsTouched();
        }

        [Test]
        public void touch_broken_left_eye_fixes_it()
        {
            GivenThereIsALeftEyeRelatedWith(BrainSector.Center);
            GivenTheLeftEyeIsBroken();
            GivenThereIsAHeadBasedOnConfig();
        
            WhenTheBrainIsTouchedIn(BrainSector.Center);
        
            ThenTheLeftEyeIsFixed();
        }

        private void GivenThereIsALeftEyeRelatedWith(BrainSector brainSector)
        {
            leftEye = new HeadPiece(HeadPieceType.LeftEye);
            headPieces.Add(brainSector, leftEye);
        }

        private void GivenTheLeftEyeIsBroken()
        {
            leftEye.Touch();
        }

        private void GivenThereIsAHeadBasedOnConfig()
        {
            var config = new InMemoryHeadConfiguration(headPieces);
            head = new Head(config); 
        }

        private void WhenTheBrainIsTouchedIn(BrainSector brainSector)
        {
            head.TouchBrainSector(brainSector);
        }

        private void ThenTheLeftEyeIsTouched()
        {
            Assert.IsTrue(leftEye.IsBroken);
        }

        private void ThenTheLeftEyeIsFixed()
        {
            Assert.IsTrue(!leftEye.IsBroken);
        }
    }
}