using System;
using System.Collections.Generic;
using Core.Actions;
using Core.Domain;
using Infrastructure;
using NSubstitute;
using NUnit.Framework;
using Presentation;

namespace Test.Unit.ACVSimulator.Presentation
{
    public class HeadPresenterTest
    {
        private HeadPresenter headPresenter;
        private RetrieveHead retrieveHeadAction;
        private TouchBrainSector touchBrainSectorAction;
        private CreateHead createHead;
        private BotAction botAction;
        private GetCurrentPoints getCurrentPoints;
        private HeadView view;

        private Head head;
        
        [SetUp]
        public void SetUp()
        {
            view = Substitute.For<HeadView>();
            retrieveHeadAction = Substitute.For<RetrieveHead>();
            touchBrainSectorAction = Substitute.For<TouchBrainSector>();
            createHead = Substitute.For<CreateHead>();
            botAction = Substitute.For<BotAction>();
            getCurrentPoints = Substitute.For<GetCurrentPoints>();

            headPresenter = new HeadPresenter(view, 
                retrieveHeadAction,
                touchBrainSectorAction,
                createHead,
                botAction,
                getCurrentPoints);
        }

        [Test]
        public void present_head()
        {
            GivenThereIsABrainWithALeftEyeMapped();
            GivenTheBrainSectorCanBeTouched();
            
            WhenHeadIsPresented();
            
            ThenHeadIsShownInView();
        }

        [Test]
        public void brain_portion_touched_present_mapped_face_part_state()
        {
            GivenThereIsABrainWithALeftEyeMapped();
            GivenTheBrainSectorCanBeTouched();
            GivenTheHeadIsPresented();

            WhenTheBrainIsTouchedIn(BrainSector.Center);

            ThenTheHeadViewShouldBeUpdated();
        }

        private void GivenTheHeadIsPresented()
        {
            WhenHeadIsPresented();
        }

        private void GivenTheBrainSectorCanBeTouched()
        {
            touchBrainSectorAction.Execute(Arg.Any<BrainSector>()).Returns(head);
        }

        private void GivenThereIsABrainWithALeftEyeMapped()
        {
            var headPieceMap = new Dictionary<BrainSector, HeadPiece>
            {
                {BrainSector.Center, new HeadPiece(HeadPieceType.LeftEye)}
            };
            
            var headConfig = new InMemoryHeadConfiguration(headPieceMap);
            head = new Head(headConfig);
            
            retrieveHeadAction.Execute().Returns(head);
        }

        private void WhenHeadIsPresented()
        {
            headPresenter.Present();
        }

        private void WhenTheBrainIsTouchedIn(object brainSector)
        {
            view.OnBrainSectorTouched += Raise.Event<Action<BrainSector>>(BrainSector.Center);
        }

        private void ThenTheHeadViewShouldBeUpdated()
        {
            view.Received().ShowHead(Arg.Any<List<HeadPiece>>());
        }

        private void ThenHeadIsShownInView()
        {
            view.Received(1).ShowHead(Arg.Is(head.GetHeadPieces()));
        }
    }
}