using System;
using System.Collections.Generic;
using Core.Domain;
using Presentation;
using Utils;

namespace Core.Actions
{
    public class BotAction
    {
        private readonly HeadRepository headRepository;
        private readonly BotConfiguration botConfiguration;

        private DateTime nextActDate;
        private int actsToUpdateInterval;
        private int currentInterval;

        public BotAction()
        {
        }

        public BotAction(HeadRepository headRepository,
            BotConfiguration botConfiguration)
        {
            this.headRepository = headRepository;
            this.botConfiguration = botConfiguration;
            
            SetNewActIntervalDate();
        }

        public virtual void Execute()
        {
            if (!ActionAvailable())
                return;
            
            var head = headRepository.Retrieve();
            List<HeadPiece> freeHeadPiece = new List<HeadPiece>();
            foreach (var headPiece in head.GetHeadPieces())
            {
                if(!headPiece.IsBroken)
                    freeHeadPiece.Add(headPiece);
            }

            if(freeHeadPiece.Count == 0)
                return;

            var targetHeadPiece = freeHeadPiece.PickRandom();
            var brainSectorToTouch = head.GetBrainSectorFor(targetHeadPiece);
            
            head.TouchBrainSector(brainSectorToTouch);

            headRepository.Save(head);

            SetNewActIntervalDate();
        }
        
        private bool ActionAvailable()
        {
            return DateTime.Now >= nextActDate;
        }

        private void SetNewActIntervalDate()
        {
            var random = new Random();
            nextActDate = DateTime.Now.AddSeconds(random.Next(1,botConfiguration.IntervalToAct()));
        }
    }
}