using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.CompilerServices;

namespace Core.Domain
{
    public class Head
    {
        private readonly Dictionary<BrainSector, HeadPiece> headPieces;
        private readonly DateTime initTime;
        
        public Head(HeadConfiguration configuration)
        {
            headPieces = configuration.GetHeadPieces();
            initTime = DateTime.Now;
        }
    
        public void TouchBrainSector(BrainSector brainSector)
        {
            if (headPieces.ContainsKey(brainSector))
                headPieces[brainSector].Touch();
            else throw new Exception("No Head piece mapped to that brain sector");
        }

        public List<HeadPiece> GetHeadPieces()
        {
            return headPieces.Values.ToList();
        }

        public BrainSector GetBrainSectorFor(HeadPiece targetHeadPiece)
        {
            foreach (var headPiece in headPieces)
            {
                if (headPiece.Value.GetHeadType() == targetHeadPiece.GetHeadType())
                    return headPiece.Key;
            }

            return BrainSector.BottomLeft;
        }

        public bool IsDead()
        {
            foreach (var headPiece in headPieces.Values)
            {
                if (!headPiece.IsBroken)
                {
                    return false;
                }
            }

            return true;
        }

        public DateTime GetInitTime()
        {
            return initTime;
        }
    }
}