using System.Collections.Generic;
using System.Linq;
using Core.Domain;
using Utils;

namespace Infrastructure.Providers
{
    public static class ConfigurationProvider
    {
        public static HeadConfiguration ProvideDefaultHeadConfiguration()
        {
            Dictionary<BrainSector, HeadPiece> headPiecesMap = new Dictionary<BrainSector, HeadPiece>();
            headPiecesMap.Add(BrainSector.BottomLeft, new HeadPiece(HeadPieceType.LeftMouth));
            headPiecesMap.Add(BrainSector.BottomRight, new HeadPiece(HeadPieceType.RightMouth));
            headPiecesMap.Add(BrainSector.Center, new HeadPiece(HeadPieceType.Nose));
            headPiecesMap.Add(BrainSector.CenterLeft, new HeadPiece(HeadPieceType.Chin));
            headPiecesMap.Add(BrainSector.CenterRight, new HeadPiece(HeadPieceType.ForeHead));
            headPiecesMap.Add(BrainSector.TopRight, new HeadPiece(HeadPieceType.RightEye));
            headPiecesMap.Add(BrainSector.TopLeft, new HeadPiece(HeadPieceType.LeftEye));
            
            return new InMemoryHeadConfiguration(headPiecesMap);
        }
        
        public static HeadConfiguration ProvideRandomHeadConfiguration()
        {
            Dictionary<BrainSector, HeadPiece> headPiecesMap = new Dictionary<BrainSector, HeadPiece>();
            
            List<BrainSector> brainSectors = new List<BrainSector>();
            brainSectors.Add(BrainSector.BottomLeft);
            brainSectors.Add(BrainSector.BottomRight);
            brainSectors.Add(BrainSector.Center);
            brainSectors.Add(BrainSector.CenterLeft);
            brainSectors.Add(BrainSector.CenterRight);
            brainSectors.Add(BrainSector.TopRight);
            brainSectors.Add(BrainSector.TopLeft);

            var shuffledBrainSectors = brainSectors.Shuffle().ToList();
            
            headPiecesMap.Add(shuffledBrainSectors[0], new HeadPiece(HeadPieceType.LeftMouth));
            headPiecesMap.Add(shuffledBrainSectors[1], new HeadPiece(HeadPieceType.RightMouth));
            headPiecesMap.Add(shuffledBrainSectors[2], new HeadPiece(HeadPieceType.Nose));
            headPiecesMap.Add(shuffledBrainSectors[3], new HeadPiece(HeadPieceType.Chin));
            headPiecesMap.Add(shuffledBrainSectors[4], new HeadPiece(HeadPieceType.ForeHead));
            headPiecesMap.Add(shuffledBrainSectors[5], new HeadPiece(HeadPieceType.RightEye));
            headPiecesMap.Add(shuffledBrainSectors[6], new HeadPiece(HeadPieceType.LeftEye));
            
            return new InMemoryHeadConfiguration(headPiecesMap);
        }

        public static BotConfiguration ProvideDefaultBotConfiguration()
        {
            return new InMemoryBotConfiguration(3);
        }
    }
}